/**
 * Narrows a type into a literal.
 */
export declare type Exact<U> = U extends any ? (any extends U ? never : U) : never;
/**
 * Narrows a type into a literal.
 */
export declare type X<U> = Exact<U>;
/**
 * Defines the union of literal falsey primitives.
 */
export declare type Falsey = false | null | undefined | 0 | 0n | '';
/**
 * Maps an outcome containing a promise to a promise containing an outcome with the promise value.
 */
export declare type Future<U> = U extends Promise<infer P> ? Promise<Outcome<P>> : Promise<Outcome<U>>;
/**
 * Represents an error outcome at compile time.
 */
export declare type Exception = {
    status: Status.Failed;
    failure: Error;
    success: Error;
    future: Future<Error>;
    onSuccess<M>(fn: (value: Error) => M): Outcome<M>;
    onFailure<M>(fn: (value: Error) => M): Quantum<M>;
};
/**
 * Represents a failure outcome at compile time.
 */
export declare type Failure<U> = {
    status: Status.Failed;
    failure: U;
    success: U;
    future: Future<U>;
    onSuccess<M>(fn: (value: U) => M): Outcome<M>;
    onFailure<M>(fn: (value: U) => M): Quantum<M>;
};
/**
 * Represents a success outcome at compile time.
 */
export declare type Success<U> = {
    status: Status.Succeeded;
    failure: U;
    success: U;
    future: Future<U>;
    onSuccess<M>(fn: (value: U) => M): Outcome<M>;
    onFailure<M>(fn: (value: U) => M): Quantum<M>;
};
/**
 * Represents an undiscovered outcome at runtime.
 */
export declare type Quantum<U> = {
    status: Status;
    failure: U;
    success: U;
    future: Future<U>;
    onSuccess<M>(fn: (value: U) => M): Outcome<M>;
    onFailure<M>(fn: (value: U) => M): Quantum<M>;
};
/**
 * Represents a future outcome at compile time.
 */
/**
 * Represents the set of all outcomes.
 */
export declare type Outcome<U> = U extends Error ? Exception : U extends Falsey ? Failure<U> : Quantum<U>;
/**
 * De-nests an outcome containing an outcome.
 */
export declare type DenestOutcome<U> = Outcome<U extends Outcome<infer X> ? X : U>;
export declare enum Status {
    Failed = 0,
    Succeeded = 1
}
